@@ -91,7 +91,7 @@ void mfs_freebsd_workarounds(int on);
 void mfs_setdisables(uint32_t disables);
 
 void mfs_term(void);
-void mfs_init (int debug_mode_in,int keep_cache_in,double direntry_cache_timeout_in,double entry_cache_timeout_in,double attr_cache_timeout_in,double xattr_cache_timeout_in,double groups_cache_timeout,int mkdir_copy_sgid_in,int sugid_clear_mode_in,int xattr_acl_support_in,double fsync_before_close_min_time_in,int no_xattrs_in,int no_posix_locks_in,int no_bsd_locks_in);
+void mfs_init (int debug_mode_in,int keep_cache_in,double readdirplus_cache_min_timeout_in,double direntry_cache_timeout_in,double entry_cache_timeout_in,double attr_cache_timeout_in,double xattr_cache_timeout_in,double groups_cache_timeout,int mkdir_copy_sgid_in,int sugid_clear_mode_in,int xattr_acl_support_in,double fsync_before_close_min_time_in,int no_xattrs_in,int no_posix_locks_in,int no_bsd_locks_in);
 
 #ifdef HAVE_FUSE3
 void mfs_setsession(struct fuse_session *se);
