@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2025 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -65,6 +65,7 @@ typedef struct old_changes_block {
 
 static old_changes_block *old_changes_head=NULL;
 static old_changes_block *old_changes_current=NULL;
+static double lastchange = 0.0;
 
 static uint64_t old_changes_total_size = 0;
 
@@ -76,6 +77,7 @@ static uint8_t ChangelogSaveMode;
 #define SAVEMODE_BACKGROUND 0
 #define SAVEMODE_ASYNC 1
 #define SAVEMODE_SYNC 2
+#define SAVEMODE_MAX 2
 
 static inline void changelog_old_changes_free_block(old_changes_block *oc) {
 	uint32_t i;
@@ -181,14 +183,20 @@ uint64_t changelog_get_minversion(void) {
 	return old_changes_head->minversion;
 }
 
-void changelog_rotate(uint8_t broadcast) {
-	if (ChangelogSaveMode==0) {
-		bgsaver_rotatelog();
-	} else {
+void changelog_rotate(uint8_t rotate_flags) {
+	if (ChangelogSaveMode!=SAVEMODE_BACKGROUND) {
+		rotate_flags |= ROTATE_FLAG_FOREGROUND;
+	}
+	if ((rotate_flags&ROTATE_FLAG_FOREGROUND)==0) { // try to rotate in background
+		if (bgsaver_rotatelog()<0) { // in case of error - switch to foreground
+			rotate_flags |= ROTATE_FLAG_FOREGROUND;
+		}
+	}
+	if (rotate_flags&ROTATE_FLAG_FOREGROUND) {
 		char logname1[100],logname2[100];
 		uint32_t i;
 		if (currentfd) {
-			if (ChangelogSaveMode==2) {
+			if (ChangelogSaveMode==SAVEMODE_SYNC) {
 				fsync(fileno(currentfd));
 			}
 			fclose(currentfd);
@@ -204,13 +212,13 @@ void changelog_rotate(uint8_t broadcast) {
 			unlink("changelog.0.mfs");
 		}
 	}
-	if (broadcast) {
+	if (rotate_flags&ROTATE_FLAG_BROADCAST) {
 		matomlserv_broadcast_logrotate();
 	}
 }
 
 void changelog_mr(uint64_t version,const char *data) {
-	if (ChangelogSaveMode==0) {
+	if (ChangelogSaveMode==SAVEMODE_BACKGROUND) {
 		bgsaver_changelog(version,data);
 	} else {
 		if (currentfd==NULL) {
@@ -223,7 +231,7 @@ void changelog_mr(uint64_t version,const char *data) {
 		if (currentfd) {
 			fprintf(currentfd,"%"PRIu64": %s\n",version,data);
 			fflush(currentfd);
-			if (ChangelogSaveMode==2) {
+			if (ChangelogSaveMode==SAVEMODE_SYNC) {
 				fsync(fileno(currentfd));
 			}
 		}
@@ -252,6 +260,7 @@ void changelog(const char *format,...) {
 
 	changelog_mr(version,printbuff);
 	changelog_store_logstring(version,(uint8_t*)printbuff,leng);
+	lastchange = monotonic_seconds();
 }
 
 char* changelog_generate_gids(uint32_t gids,uint32_t *gid) {
@@ -323,6 +332,40 @@ char* changelog_escape_name(uint32_t nleng,const uint8_t *name) {
 	return currescname;
 }
 
+void changelog_sendnop(void) {
+	if (lastchange + 30.0 <= monotonic_seconds()) {
+		changelog("%"PRIu32"|IDLE()",main_time());
+	}
+}
+
+void changelog_info(FILE *fd) {
+	uint64_t minversion_to_keep;
+	uint64_t current_meteversion;
+
+	current_meteversion = meta_version();
+
+	fprintf(fd,"[changelog]\n");
+	if (old_changes_head) {
+		fprintf(fd,"oldest_metaversion: %"PRIu64"\n",old_changes_head->minversion);
+		fprintf(fd,"oldest_timestamp: %"PRIu32" (%d seconds)\n",old_changes_head->mintimestamp,main_time()-old_changes_head->mintimestamp);
+		fprintf(fd,"total_size: %"PRIu64"\n",old_changes_total_size);
+	} else {
+		fprintf(fd,"changelog memory is empty\n");
+	}
+	minversion_to_keep = meta_chlog_keep_version();
+	if (minversion_to_keep < current_meteversion) {
+		fprintf(fd,"min_changelog_kept_for_metadata_sending: %"PRIu64"\n",minversion_to_keep);
+	} else {
+		fprintf(fd,"min_changelog_kept_for_metadata_sending: -\n");
+	}
+	minversion_to_keep = matomlserv_get_min_version();
+	if (minversion_to_keep < current_meteversion) {
+		fprintf(fd,"min_changelog_kept_for_delayed_receivers: %"PRIu64"\n",minversion_to_keep);
+	} else {
+		fprintf(fd,"min_changelog_kept_for_delayed_receivers: -\n");
+	}
+	fprintf(fd,"\n");
+}
 
 void changelog_reload(void) {
 	uint16_t changelog_preserve_mb;
@@ -348,15 +391,17 @@ void changelog_reload(void) {
 	ChangeLogMaxSize = changelog_preserve_mb;
 	ChangeLogMaxSize *= (1024*1024);
 	ChangelogSaveMode = cfg_getuint8("CHANGELOG_SAVE_MODE",0);
-	if (ChangelogSaveMode>2) {
+	if (ChangelogSaveMode>SAVEMODE_MAX) {
 		mfs_log(MFSLOG_SYSLOG_STDERR,MFSLOG_WARNING,"CHANGELOG_SAVE_MODE - wrong value - using 0 (write in background)");
-		ChangelogSaveMode = 0;
+		ChangelogSaveMode = SAVEMODE_BACKGROUND;
 	}
 }
 
 int changelog_init(void) {
 	changelog_reload();
+	main_time_register(1,0,changelog_sendnop);
 	main_reload_register(changelog_reload);
+	main_info_register(changelog_info);
 	currentfd = NULL;
 	return 0;
 }
