@@ -369,14 +369,14 @@ static inline void flock_lock_unlock(inodelocks *il,lock *l) {
 void flock_disconnected(void *connptr) {
 	uint32_t h;
 	inodelocks *il;
-	lock *l,*nl;
+	lock *l,**lptr;
 	instance *i,*ni;
 	for (h=0 ; h<FLOCK_INODE_HASHSIZE ; h++) {
 		for (il = inodehash[h] ; il!=NULL ; il=il->next) {
-			l = il->waiting_head;
-			while (l!=NULL) {
-				nl = l->next;
+			lptr = &(il->waiting_head);
+			while ((l=*lptr)!=NULL) {
 				if (l->connptr==connptr) {
+					*lptr = l->next;
 					i = l->lock_instances;
 					while (i) {
 						ni = i->next;
@@ -384,8 +384,9 @@ void flock_disconnected(void *connptr) {
 						i = ni;
 					}
 					free(l);
+				} else {
+					lptr = &(l->next);
 				}
-				l = nl;
 			}
 		}
 	}
@@ -574,13 +575,13 @@ uint32_t flock_list(uint32_t inode,uint8_t *buff) {
 						buff+=20;
 						switch (l->ltype) {
 							case LTYPE_READER:
-								put8bit(&buff,1);
+								put8bit(&buff,MFS_LOCK_TYPE_SHARED);
 								break;
 							case LTYPE_WRITER:
-								put8bit(&buff,2);
+								put8bit(&buff,MFS_LOCK_TYPE_EXCLUSIVE);
 								break;
 							default:
-								put8bit(&buff,0);
+								put8bit(&buff,MFS_LOCK_TYPE_UNKNOWN);
 						}
 					}
 				}
@@ -599,13 +600,13 @@ uint32_t flock_list(uint32_t inode,uint8_t *buff) {
 					buff+=20;
 					switch (l->ltype) {
 						case LTYPE_READER:
-							put8bit(&buff,1);
+							put8bit(&buff,MFS_LOCK_TYPE_SHARED);
 							break;
 						case LTYPE_WRITER:
-							put8bit(&buff,2);
+							put8bit(&buff,MFS_LOCK_TYPE_EXCLUSIVE);
 							break;
 						default:
-							put8bit(&buff,0);
+							put8bit(&buff,MFS_LOCK_TYPE_UNKNOWN);
 					}
 				}
 			}
